#ifndef RT_VK_SEMAPHORES_H
#define RT_VK_SEMAPHORES_H

#include <volk/volk.h>
#include <runtime/threading.h>

struct rt_vk_device;

typedef struct {
    /* A timeline semaphore */
    VkSemaphore semaphore;

    /* It's current value, or the next value it will signal */
    uint64_t value;

    rt_bool32 in_use;
} rt_vk_semaphore;

typedef struct {
    struct rt_vk_device *dev;

    rt_vk_semaphore *semaphores;

    /* Number of semaphores inside the pool */
    uint32_t size;

    /* Running index of acquire operations. Atomically incremented to fetch a "new" semaphore */
    uint32_t acquire_index;

    /* Locked as writing when resizing the array. During normal usage, this gets a read lock */
    rt_rwlock resize_lock;
} rt_vk_semaphore_pool;

typedef struct {
    rt_result result;
    rt_vk_semaphore_pool semaphore_pool;
} rt_create_vk_semaphore_pool_result;

rt_create_vk_semaphore_pool_result rtCreateVkSemaphorePool(struct rt_vk_device *device, uint32_t initial_size);

void rtDestroyVkSemaphorePool(rt_vk_semaphore_pool *pool);

rt_vk_semaphore *rtAcquireSemaphore(rt_vk_semaphore_pool *pool);

void rtReleaseSemaphore(rt_vk_semaphore *semaphore);

#endif
